/********************
 * 2004 Jalview Reengineered
 * Barton Group
 * Dundee University
 *
 * AM Waterhouse
 *******************/




package jalview.appletgui;

import jalview.jbappletgui.GAlignFrame;
import jalview.schemes.*;
import jalview.datamodel.*;
import jalview.analysis.*;
import jalview.io.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;


public class AlignFrame extends GAlignFrame
{
  final AlignmentPanel alignPanel;
  final AlignViewport  viewport;
  public static final int NEW_WINDOW_WIDTH = 700;
  public static final int NEW_WINDOW_HEIGHT = 500;
  jalview.bin.JalviewLite applet;

  public AlignFrame(AlignmentI al)
  {
    this.applet = null;
    viewport = new AlignViewport(al, null);
    alignPanel = new AlignmentPanel(this, viewport);
    add(alignPanel);
    alignPanel.validate();
  }

  public AlignFrame(AlignmentI al, jalview.bin.JalviewLite applet)
  {
    this.applet = applet;
    viewport = new AlignViewport(al, applet);
    alignPanel = new AlignmentPanel(this, viewport);
    add(alignPanel);
    alignPanel.validate();

    String treeFile = applet.getParameter("treeFile");
    TreePanel treepanel = null;
    if (treeFile != null)
    {
      try
      {
        treeFile = treeFile;
        jalview.io.NewickFile fin = new jalview.io.NewickFile( applet.getCodeBase()+treeFile, "URL");
        fin.parse();

        if (fin.getTree() != null)
        {
          TreePanel tp = null;
          tp = new TreePanel(viewport, viewport.getAlignment().getSequences(),
                             fin, "FromFile", applet.getCodeBase()+treeFile);
          jalview.bin.JalviewLite.addFrame(tp, treeFile, 600, 500);
          addTreeMenuItem(tp, treeFile);
        }
      }
      catch (Exception ex)
      {
        ex.printStackTrace();
      }
    }


  }

  public void inputText_actionPerformed(ActionEvent e)
  {
     CutAndPasteTransfer cap = new CutAndPasteTransfer(true);
     Frame frame = new Frame();
     frame.add(cap);
     jalview.bin.JalviewLite.addFrame(frame, "Cut & Paste Input", 500,500);
  }


  protected void outputText_actionPerformed(ActionEvent e)
  {
     CutAndPasteTransfer cap = new CutAndPasteTransfer(false);
     Frame frame = new Frame();
     frame.add(cap);
     jalview.bin.JalviewLite.addFrame(frame,"Alignment output - "+e.getActionCommand(),600, 500);
     cap.setText( FormatAdapter.formatSequences(e.getActionCommand(), viewport.getAlignment().getSequences()));
  }

  public void closeMenuItem_actionPerformed(ActionEvent e)
  {
    dispose();
  }

  Stack historyList = new Stack();
  Stack redoList = new Stack();

  void updateEditMenuBar()
  {
    if(historyList.size()>0)
     {
       undoMenuItem.setEnabled(true);
       Object [] history = (Object[])historyList.elementAt(0);
       undoMenuItem.setLabel("Undo "+history[0]);
     }
    else
    {
      undoMenuItem.setEnabled(false);
      undoMenuItem.setLabel("Undo");
    }

    if(redoList.size()>0)
     {
       redoMenuItem.setEnabled(true);
       Object [] history = (Object[])redoList.elementAt(0);
       redoMenuItem.setLabel("Redo "+history[0]);
     }
    else
    {
      redoMenuItem.setEnabled(false);
      redoMenuItem.setLabel("Redo");
    }
  }

  public void addHistoryItem(String type)
  {
    // must make sure we add new sequence objects her, not refs to the existing sequences
    redoList.removeAllElements();
    SequenceI[] seq = new SequenceI[viewport.getAlignment().getHeight()];
    for(int i=0; i<viewport.getAlignment().getHeight(); i++)
    {
      seq[i] = new Sequence( viewport.getAlignment().getSequenceAt(i).getName(),
                             viewport.getAlignment().getSequenceAt(i).getSequence());
    }

    historyList.addElement(new Object[]{type,  seq} );
    updateEditMenuBar();
  }

  protected void undoMenuItem_actionPerformed(ActionEvent e)
  {
    Object [] history = (Object[])historyList.elementAt(0);
    historyList.removeElementAt(0);
    // add the redo state before continuing!!
    SequenceI[] seq = new SequenceI[viewport.getAlignment().getHeight()];
    for (int i = 0; i < viewport.getAlignment().getHeight(); i++)
    {
      seq[i] = new Sequence(viewport.getAlignment().getSequenceAt(i).getName(),
                            viewport.getAlignment().getSequenceAt(i).
                            getSequence());
    }
    /////////

    redoList.addElement(new Object[] {history[0], seq});

      seq = (SequenceI[]) history[1];

      AlignmentAnnotation[] old = viewport.alignment.getAlignmentAnnotation();
      viewport.setAlignment(new Alignment(seq));
      updateEditMenuBar();
      for (int i = 0; i < old.length; i++)
        viewport.alignment.addAnnotation(old[i]);

      viewport.updateConservation();
      viewport.updateConsensus();
      alignPanel.repaint();
  }

  public void moveSelectedSequences(boolean up)
  {
    SequenceGroup sg = viewport.getSelectionGroup();
    if (sg == null)
      return;

    if (up)
    {
      for (int i = 1; i < viewport.alignment.getHeight(); i++)
      {
        SequenceI seq = viewport.alignment.getSequenceAt(i);
        if (!sg.sequences.contains(seq))
          continue;

        SequenceI temp = viewport.alignment.getSequenceAt(i - 1);
        if (sg.sequences.contains(temp))
          continue;

        viewport.alignment.getSequences().setElementAt(temp, i);
        viewport.alignment.getSequences().setElementAt(seq, i - 1);
      }
    }
    else
    {
      for (int i = viewport.alignment.getHeight() - 2; i > -1; i--)
      {
        SequenceI seq = viewport.alignment.getSequenceAt(i);
        if (!sg.sequences.contains(seq))
          continue;

        SequenceI temp = viewport.alignment.getSequenceAt(i + 1);
        if (sg.sequences.contains(temp))
          continue;

        viewport.alignment.getSequences().setElementAt(temp, i);
        viewport.alignment.getSequences().setElementAt(seq, i + 1);
      }
    }

    alignPanel.repaint();
  }


  StringBuffer copiedSequences;
  protected void copy_actionPerformed(ActionEvent e)
  {
   if(viewport.getSelectionGroup()==null)
     return;

   SequenceGroup sg = viewport.getSelectionGroup();
   copiedSequences = new StringBuffer();
   Hashtable orderedSeqs = new Hashtable();
   for(int i=0; i<sg.getSize(); i++)
   {
      SequenceI seq = sg.getSequenceAt(i);
      int index = viewport.alignment.findIndex(seq);
      orderedSeqs.put(index+"", seq);
   }

   int index=0;
   for(int i=0; i<sg.getSize(); i++)
   {
     SequenceI seq = null;
     while( seq == null )
     {
       if(orderedSeqs.containsKey(index+""))
       {
         seq = (SequenceI) orderedSeqs.get(index + "");
         index++;
         break;
       }
       else
         index++;
     }

         copiedSequences.append( seq.getName()+"\t"+seq.findPosition( sg.getStartRes() ) +"\t"
                        +seq.findPosition( sg.getEndRes() )+ "\t"
                        +sg.getSequenceAt(i).getSequence(sg.getStartRes(), sg.getEndRes()+1)+"\n");
       }

  }


  protected void pasteNew_actionPerformed(ActionEvent e)
  {
    paste(true);
  }

  protected void pasteThis_actionPerformed(ActionEvent e)
  {
    addHistoryItem("Paste");
    paste(false);
  }

  void paste(boolean newAlignment)
  {
    try{
      if (copiedSequences == null)
        return;

      StringTokenizer st = new StringTokenizer(copiedSequences.toString());
      Vector seqs = new Vector();
      while (st.hasMoreElements())
      {
        String name = st.nextToken();
        int start = Integer.parseInt(st.nextToken());
        int end = Integer.parseInt(st.nextToken());
        Sequence sequence = new Sequence(name,st.nextToken(), start, end);

        if(!newAlignment)
          viewport.alignment.addSequence(sequence);
        else
          seqs.addElement(sequence);
      }

      if(newAlignment)
      {
        SequenceI[] newSeqs = new SequenceI[seqs.size()];
        for(int i=0; i<seqs.size(); i++)
         newSeqs[i] = (SequenceI)seqs.elementAt(i);

        AlignFrame af = new AlignFrame(new Alignment(newSeqs), applet);
        String newtitle = new String("Copied sequences");
        if( getTitle().startsWith("Copied sequences"))
         newtitle = getTitle();
       else
         newtitle = newtitle.concat("- from "+getTitle());
        jalview.bin.JalviewLite.addFrame(af, newtitle, NEW_WINDOW_WIDTH, NEW_WINDOW_HEIGHT);
      }
      else
      {
        viewport.setEndSeq(viewport.alignment.getHeight());
        viewport.alignment.getWidth();
        viewport.updateConservation();
        viewport.updateConsensus();
        alignPanel.repaint();
      }

    }catch(Exception ex){}// could be anything being pasted in here

  }


  protected void cut_actionPerformed(ActionEvent e)
  {
    copy_actionPerformed(null);
    delete_actionPerformed(null);
  }

  protected void delete_actionPerformed(ActionEvent e)
  {
    addHistoryItem("Delete");
    if (viewport.getSelectionGroup() == null)
      return;

     SequenceGroup sg = viewport.getSelectionGroup();
     for (int i=0;i < sg.sequences.size(); i++)
     {
       SequenceI seq = sg.getSequenceAt(i);
       int index = viewport.getAlignment().findIndex(seq);
       seq.deleteChars(sg.getStartRes(), sg.getEndRes()+1);

       if(seq.getSequence().length()<1)
          viewport.getAlignment().deleteSequence(seq);
      else
          viewport.getAlignment().getSequences().setElementAt(seq, index);
     }

     viewport.setSelectionGroup(null);
     viewport.alignment.deleteGroup(sg);
     viewport.resetSeqLimits( alignPanel.seqPanel.seqCanvas.getSize().height);
     if(viewport.getAlignment().getHeight()<1)
     try
     {
       this.setVisible(false);
     }catch(Exception ex){}
   viewport.updateConservation();
   viewport.updateConsensus();
     alignPanel.repaint();

  }



  protected void redoMenuItem_actionPerformed(ActionEvent e)
  {
     Object [] history = (Object[])redoList.pop();
     SequenceI[] seq = (SequenceI[]) history[1];

     AlignmentAnnotation[] old = viewport.alignment.getAlignmentAnnotation();
     viewport.setAlignment(new Alignment(seq));
     for (int i = 0; i < old.length; i++)
       viewport.alignment.addAnnotation(old[i]);

     viewport.updateConservation();
     viewport.updateConsensus();
     updateEditMenuBar();
     alignPanel.repaint();
  }


  protected void deleteGroups_actionPerformed(ActionEvent e)
  {
    viewport.alignment.deleteAllGroups();
    viewport.setSelectionGroup(null);

    alignPanel.repaint();
  }



  public void selectAllSequenceMenuItem_actionPerformed(ActionEvent e)
  {
    SequenceGroup sg = new SequenceGroup();
    for (int i=0; i<viewport.getAlignment().getSequences().size(); i++)
      sg.addSequence( viewport.getAlignment().getSequenceAt(i));
    sg.setEndRes(viewport.alignment.getWidth());
    viewport.setSelectionGroup(sg);
    alignPanel.repaint();
    PaintRefresher.Refresh(null);
  }

  public void deselectAllSequenceMenuItem_actionPerformed(ActionEvent e)
  {
    viewport.setSelectionGroup(null);
    viewport.getColumnSelection().clear();
    viewport.setSelectionGroup(null);
    PaintRefresher.Refresh(null);
  }

  public void invertSequenceMenuItem_actionPerformed(ActionEvent e)
  {
    SequenceGroup sg = viewport.getSelectionGroup();
    for (int i=0; i<viewport.getAlignment().getSequences().size(); i++)
      sg.addOrRemove (viewport.getAlignment().getSequenceAt(i));

    PaintRefresher.Refresh(null);
  }

  public void remove2LeftMenuItem_actionPerformed(ActionEvent e)
  {
    addHistoryItem("delete columns");
    ColumnSelection colSel = viewport.getColumnSelection();
    if (colSel.size() > 0)
    {
      int min = colSel.getMin();
      viewport.getAlignment().trimLeft(min);
      colSel.compensateForEdit(0,min);

      if(viewport.getSelectionGroup()!=null)
        viewport.getSelectionGroup().adjustForRemoveLeft(min);

      Vector groups = viewport.alignment.getGroups();
      for(int i=0; i<groups.size(); i++)
      {
        SequenceGroup sg = (SequenceGroup) groups.elementAt(i);
        if(!sg.adjustForRemoveLeft(min))
          viewport.alignment.deleteGroup(sg);
      }

      alignPanel.repaint();
    }
  }

  public void remove2RightMenuItem_actionPerformed(ActionEvent e)
  {
    addHistoryItem("delete columns");
    ColumnSelection colSel = viewport.getColumnSelection();
    if (colSel.size() > 0)
    {
      int max = colSel.getMax();
      viewport.getAlignment().trimRight(max);
      if(viewport.getSelectionGroup()!=null)
        viewport.getSelectionGroup().adjustForRemoveRight(max);

      Vector groups = viewport.alignment.getGroups();
      for(int i=0; i<groups.size(); i++)
      {
        SequenceGroup sg = (SequenceGroup) groups.elementAt(i);
        if(!sg.adjustForRemoveRight(max))
          viewport.alignment.deleteGroup(sg);
      }



      alignPanel.repaint();
    }

  }

  public void removeGappedColumnMenuItem_actionPerformed(ActionEvent e)
  {
    addHistoryItem("delete gapped columns");
    viewport.getAlignment().removeGaps();
    viewport.updateConservation();
    viewport.updateConsensus();
    alignPanel.repaint();
  }

  public void removeAllGapsMenuItem_actionPerformed(ActionEvent e)
  {
    addHistoryItem("delete all gaps");
    SequenceI current;
    int jSize;
    for (int i=0; i < viewport.getAlignment().getSequences().size();i++)
    {
      current = viewport.getAlignment().getSequenceAt(i);
      jSize = current.getLength();
      for (int j=0; j < jSize; j++)
        if(jalview.util.Comparison.isGap(current.getCharAt(j)))
        {
          current.deleteCharAt(j);
          j--;
          jSize--;
        }
    }
    viewport.updateConservation();
    viewport.updateConsensus();
    alignPanel.repaint();
  }


  public void findMenuItem_actionPerformed(ActionEvent e)
  {
    Finder finder = new Finder(alignPanel);
  }


  public void font_actionPerformed(ActionEvent e)
  {
    Frame frame = new Frame();
    FontChooser fc = new FontChooser( alignPanel, frame);
    frame.add(fc);
    jalview.bin.JalviewLite.addFrame(frame, "Change Font", 440,100);

  }

  protected void fullSeqId_actionPerformed(ActionEvent e)
  {
    viewport.setShowFullId( fullSeqId.getState() );
    alignPanel.idPanel.idCanvas.setSize( alignPanel.calculateIdWidth() );
    alignPanel.repaint();
  }

  protected void colourTextMenuItem_actionPerformed(ActionEvent e)
  {
      viewport.setColourText( colourTextMenuItem.getState() );
      alignPanel.repaint();
  }

  protected void wrapMenuItem_actionPerformed(ActionEvent e)
  {
    viewport.setWrapAlignment( wrapMenuItem.getState() );
    alignPanel.setWrapAlignment( wrapMenuItem.getState() );
    scaleAbove.setEnabled( wrapMenuItem.getState() );
    scaleLeft.setEnabled( wrapMenuItem.getState());
    scaleRight.setEnabled( wrapMenuItem.getState());
    alignPanel.repaint();
  }

  protected void scaleAbove_actionPerformed(ActionEvent e)
  {
    viewport.setScaleAboveWrapped(scaleAbove.getState());
    alignPanel.repaint();
  }

  protected void scaleLeft_actionPerformed(ActionEvent e)
  {
    viewport.setScaleLeftWrapped(scaleLeft.getState());
    alignPanel.repaint();
  }

  protected void scaleRight_actionPerformed(ActionEvent e)
  {
    viewport.setScaleRightWrapped(scaleRight.getState());
    alignPanel.repaint();
  }



  public void viewBoxesMenuItem_actionPerformed(ActionEvent e)
  {
    viewport.setShowBoxes( viewBoxesMenuItem.getState() );
    alignPanel.repaint();
  }

  public void viewTextMenuItem_actionPerformed(ActionEvent e)
  {
    viewport.setShowText( viewTextMenuItem.getState() );
    alignPanel.repaint();
  }


  protected void renderGapsMenuItem_actionPerformed(ActionEvent e)
  {
    viewport.setRenderGaps(renderGapsMenuItem.getState());
    alignPanel.repaint();
  }


  public void annotationPanelMenuItem_actionPerformed(ActionEvent e)
  {
    if(annotationPanelMenuItem.getState() && viewport.getWrapAlignment())
    {
      annotationPanelMenuItem.setState(false);
      return;
    }
    viewport.setShowAnnotation( annotationPanelMenuItem.getState() );
    alignPanel.setAnnotationVisible( annotationPanelMenuItem.getState() );
  }

  public void overviewMenuItem_actionPerformed(ActionEvent e)
  {
   if (alignPanel.overviewPanel != null)
      return;

    Frame frame = new Frame();
    OverviewPanel overview = new OverviewPanel(alignPanel);
    frame.add(overview);
    // +50 must allow for applet frame window
    jalview.bin.JalviewLite.addFrame(frame, "Overview " + this.getTitle(),
                             overview.preferredSize().width, overview.preferredSize().height+50);

    frame.pack();
    frame.addWindowListener(new WindowAdapter()
    { public void windowClosing(WindowEvent e)
      {
            alignPanel.setOverviewPanel(null);
      };
    });

    alignPanel.setOverviewPanel( overview );


  }

  protected void noColourmenuItem_actionPerformed(ActionEvent e)
  {
    changeColour( null );
  }


  public void clustalColour_actionPerformed(ActionEvent e)
  {
    abovePIDThreshold.setState(false);
    changeColour(new ClustalxColourScheme(viewport.alignment.getSequences(), viewport.alignment.getWidth()));
  }

  public void zappoColour_actionPerformed(ActionEvent e)
  {
    changeColour(new ZappoColourScheme());
  }

  public void taylorColour_actionPerformed(ActionEvent e)
  {
    changeColour(new TaylorColourScheme());
  }


  public void hydrophobicityColour_actionPerformed(ActionEvent e)
  {
    changeColour( new HydrophobicColourScheme() );
  }

  public void helixColour_actionPerformed(ActionEvent e)
  {
    changeColour(new HelixColourScheme() );
  }


  public void strandColour_actionPerformed(ActionEvent e)
  {
    changeColour(new StrandColourScheme());
  }


  public void turnColour_actionPerformed(ActionEvent e)
  {
    changeColour(new TurnColourScheme());
  }


  public void buriedColour_actionPerformed(ActionEvent e)
  {
    changeColour(new BuriedColourScheme() );
  }

  public void nucleotideColour_actionPerformed(ActionEvent e)
  {
    changeColour(new NucleotideColourScheme());
  }


  protected void applyToAllGroups_actionPerformed(ActionEvent e)
  {
    viewport.setColourAppliesToAllGroups(applyToAllGroups.getState());
  }



  void changeColour(ColourSchemeI cs)
  {
    int threshold = 0;

    if ( viewport.getAbovePIDThreshold() )
    {
      threshold = SliderPanel.setPIDSliderSource(alignPanel, cs, "Background");

      if (cs instanceof ResidueColourScheme)
        ( (ResidueColourScheme) cs).setThreshold(threshold);
      else if (cs instanceof ScoreColourScheme)
        ( (ScoreColourScheme) cs).setThreshold(threshold);

      viewport.setGlobalColourScheme(cs);
    }
    else if (cs instanceof ResidueColourScheme)
      ( (ResidueColourScheme) cs).setThreshold(0);
    else if (cs instanceof ScoreColourScheme)
      ( (ScoreColourScheme) cs).setThreshold(0);



    if (viewport.getConservationSelected())
    {
      ConservationColourScheme ccs = null;

      Alignment al = (Alignment) viewport.alignment;
      Conservation c = new Conservation("All",
                                        ResidueProperties.propHash, 3,
                                        al.getSequences(), 0,
                                        al.getWidth() - 1);

      c.calculate();
      c.verdict(false, viewport.ConsPercGaps);

      ccs = new ConservationColourScheme(c, cs);

      // MUST NOTIFY THE COLOURSCHEME OF CONSENSUS!
      ccs.setConsensus( viewport.vconsensus );
      viewport.setGlobalColourScheme(ccs);

      SliderPanel.setConservationSlider(alignPanel, ccs, "Background");

    }
    else
    {
        // MUST NOTIFY THE COLOURSCHEME OF CONSENSUS!
        if (cs != null)
          cs.setConsensus(viewport.vconsensus);
        viewport.setGlobalColourScheme(cs);
    }


    if(viewport.getColourAppliesToAllGroups())
    {
      Vector groups = viewport.alignment.getGroups();
      for(int i=0; i<groups.size(); i++)
      {
        SequenceGroup sg = (SequenceGroup)groups.elementAt(i);

        if (cs instanceof ClustalxColourScheme)
        {
          sg.cs = new ClustalxColourScheme(sg.sequences, sg.getWidth());
        }
        else
          try
          {
            sg.cs = (ColourSchemeI) cs.getClass().newInstance();
          }
          catch (Exception ex)
          {
            ex.printStackTrace();
            sg.cs = cs;
          }

        if(viewport.getAbovePIDThreshold())
        {
          if (sg.cs instanceof ResidueColourScheme)
            ( (ResidueColourScheme) sg.cs).setThreshold(threshold);
          else if (sg.cs instanceof ScoreColourScheme)
            ( (ScoreColourScheme) sg.cs).setThreshold(threshold);

           sg.cs.setConsensus( AAFrequency.calculate(sg.sequences, 0, sg.getWidth()) );
        }

        if( viewport.getConservationSelected() )
        {
          Conservation c = new Conservation("Group",
                                            ResidueProperties.propHash, 3,
                                            sg.sequences, 0, viewport.alignment.getWidth()-1);
          c.calculate();
          c.verdict(false, viewport.ConsPercGaps);
          ConservationColourScheme ccs = new ConservationColourScheme(c, sg.cs);

          // MUST NOTIFY THE COLOURSCHEME OF CONSENSUS!
          ccs.setConsensus( AAFrequency.calculate(sg.sequences, 0, sg.getWidth()));
          sg.cs = ccs;
        }
        else
        {
          // MUST NOTIFY THE COLOURSCHEME OF CONSENSUS!
          sg.cs.setConsensus(AAFrequency.calculate(sg.sequences, 0, sg.getWidth()));
        }

      }
    }

    if(alignPanel.getOverviewPanel()!=null)
      alignPanel.getOverviewPanel().updateOverviewImage();

    alignPanel.repaint();
  }

  protected void modifyPID_actionPerformed(ActionEvent e)
  {
      if(viewport.getAbovePIDThreshold())
      {
        SliderPanel.setPIDSliderSource(alignPanel, viewport.getGlobalColourScheme(),
                                   "Background");
        SliderPanel.showPIDSlider();
      }
  }

  protected void modifyConservation_actionPerformed(ActionEvent e)
  {
    if(viewport.getConservationSelected())
    {
      SliderPanel.setConservationSlider(alignPanel, viewport.globalColourScheme,
                                        "Background");
      SliderPanel.showConservationSlider();
    }
  }


  protected  void conservationMenuItem_actionPerformed(ActionEvent e)
  {
    viewport.setConservationSelected(conservationMenuItem.getState());

    viewport.setAbovePIDThreshold(false);
    abovePIDThreshold.setState(false);

   ColourSchemeI cs = viewport.getGlobalColourScheme();
   if(cs instanceof ConservationColourScheme )
     changeColour( ((ConservationColourScheme)cs).cs );
    else
      changeColour( cs );

    modifyConservation_actionPerformed(null);
  }

  public void abovePIDThreshold_actionPerformed(ActionEvent e)
  {
    viewport.setAbovePIDThreshold(abovePIDThreshold.getState());

    conservationMenuItem.setState(false);
    viewport.setConservationSelected(false);

    ColourSchemeI cs = viewport.getGlobalColourScheme();

    if(cs instanceof ConservationColourScheme )
        changeColour( ((ConservationColourScheme)cs).cs );
    else
        changeColour( cs );

    modifyPID_actionPerformed(null);
  }



  public void userDefinedColour_actionPerformed(ActionEvent e)
  {
    UserDefinedColours chooser = new UserDefinedColours( alignPanel, null);
  }

  public void PIDColour_actionPerformed(ActionEvent e)
  {
    changeColour( new PIDColourScheme() );
  }


  public void BLOSUM62Colour_actionPerformed(ActionEvent e)
  {
    changeColour(new Blosum62ColourScheme() );
  }



  public void sortPairwiseMenuItem_actionPerformed(ActionEvent e)
  {
    addHistoryItem("sort");
    AlignmentSorter.sortByPID(viewport.getAlignment(), viewport.getAlignment().getSequenceAt(0));
    alignPanel.repaint();
  }

  public void sortIDMenuItem_actionPerformed(ActionEvent e)
  {
    addHistoryItem("sort");
    AlignmentSorter.sortByID( viewport.getAlignment() );
    alignPanel.repaint();
  }

  public void sortGroupMenuItem_actionPerformed(ActionEvent e)
  {
    addHistoryItem("sort");
    AlignmentSorter.sortByGroup(viewport.getAlignment());
    AlignmentSorter.sortGroups(viewport.getAlignment());
    alignPanel.repaint();
  }

  public void removeRedundancyMenuItem_actionPerformed(ActionEvent e)
  {
    RedundancyPanel sp = new RedundancyPanel(alignPanel);
    Frame frame = new Frame();
    frame.add(sp);
    jalview.bin.JalviewLite.addFrame(frame, "Redundancy threshold selection", 400, 120);

  }

  public void pairwiseAlignmentMenuItem_actionPerformed(ActionEvent e)
  {
    if(viewport.getSelectionGroup().getSize()>1)
    {
      Frame frame = new Frame();
      frame.add(new PairwiseAlignPanel(viewport));
      jalview.bin.JalviewLite.addFrame(frame, "Pairwise Alignment", 600, 500);
    }
  }

  public void PCAMenuItem_actionPerformed(ActionEvent e)
  {

    if( (viewport.getSelectionGroup()!=null && viewport.getSelectionGroup().getSize()<4 && viewport.getSelectionGroup().getSize()>0)
       || viewport.getAlignment().getHeight()<4)
    {
      return;
    }

    try{
      PCAPanel pcaPanel = new PCAPanel(viewport, null);
      Frame frame = new Frame();
      frame.add(pcaPanel);
      jalview.bin.JalviewLite.addFrame(frame, "Principal component analysis", 400, 400);
   }catch(java.lang.OutOfMemoryError ex)
   {
   }


  }

  public void averageDistanceTreeMenuItem_actionPerformed(ActionEvent e)
  {
    NewTreePanel("AV", "PID", "Average distance tree using PID");
  }

  public void neighbourTreeMenuItem_actionPerformed(ActionEvent e)
  {
    NewTreePanel("NJ", "PID", "Neighbour joining tree using PID");
  }


  protected void njTreeBlosumMenuItem_actionPerformed(ActionEvent e)
  {
    NewTreePanel("NJ", "BL", "Neighbour joining tree using BLOSUM62");
  }

  protected void avTreeBlosumMenuItem_actionPerformed(ActionEvent e)
  {
    NewTreePanel("AV", "BL", "Average distance tree using BLOSUM62PID");
  }

  void NewTreePanel(String type, String pwType, String title)
  {
    //are the sequences aligned?
    if(!viewport.alignment.isAligned())
    {
      return;
    }

    final TreePanel tp;
    if (viewport.getSelectionGroup() != null &&
        viewport.getSelectionGroup().getSize() > 3)
    {
      tp = new TreePanel(viewport, viewport.getSelectionGroup().sequences, type,
                         pwType,
                         0, viewport.alignment.getWidth());
    }
    else
    {
      tp = new TreePanel(viewport, viewport.getAlignment().getSequences(),
                         type, pwType, 0, viewport.alignment.getWidth());
    }

   addTreeMenuItem(tp, title);

   jalview.bin.JalviewLite.addFrame(tp, title, 600, 500);
  }

  void addTreeMenuItem(final TreePanel treePanel, String title)
  {
    final MenuItem item = new MenuItem(title);
    sortByTreeMenu.add(item);
    item.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        addHistoryItem("sort");
        AlignmentSorter.sortByTree(viewport.getAlignment(), treePanel.getTree());
        alignPanel.repaint();
      }
    });

    treePanel.addWindowListener(new WindowAdapter()
    {
      public void windowClosing(WindowEvent e)
      {
        sortByTreeMenu.remove(item);
      };
    });
  }

  protected void documentation_actionPerformed(ActionEvent e) {

    jalview.bin.JalviewLite.showURL("http://www.jalview.org");

  }

  protected void about_actionPerformed(ActionEvent e)
  {
    class AboutPanel extends Canvas
    {
      public void paint(Graphics g)
      {
        g.setColor(Color.white);
        g.fillRect(0,0,getSize().width, getSize().height);
        g.setFont(new Font("Helvetica", Font.PLAIN, 12));
        FontMetrics fm = g.getFontMetrics();
        int fh = fm.getHeight();
        int y=20, x=7;
        g.setColor(Color.black);
        g.setFont(new Font("Helvetica", Font.BOLD, 14));
        g.drawString("JalView - Release 2.0", 200,y+=fh);
        g.setFont(new Font("Helvetica", Font.PLAIN, 12));
        g.drawString("Authors:  Michele Clamp, James Cuff, Steve Searle, Andrew Waterhouse, Jim Procter & Geoff Barton.",x,y+=fh*2);
        g.drawString("Current development managed by Andrew Waterhouse; Barton Group, University of Dundee.",x,y+=fh);
        g.drawString("If  you use JalView, please cite:",x,y+=fh+8);
        g.drawString("\"Clamp, M., Cuff, J., Searle, S. M. and Barton, G. J. (2004), The Jalview Java Alignment Editor\"",x,y+=fh);
        g.drawString("Bioinformatics,  2004 12;426-7.",x,y+=fh);
      }
    }
    Frame frame = new Frame();
    frame.add(new AboutPanel());
    jalview.bin.JalviewLite.addFrame(frame, "Jalview", 580,200);



  }
}
